//
//  ReactionDatasource.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 12/30/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import AmitySDK

protocol IReactionDatasource {
    var dataSourceObserver: DataSourceListener? { get set }
    func numberOfReactions() -> Int
    func reaction(for indexPath: IndexPath) -> ReactionViewModel
    func loadMore()
}

struct ReactionViewModel {
    let reactionName: String
    let userDisplayName: String
    let createdAt: Date?
}

class LiveReactionDatasource: IReactionDatasource {
    
    weak var dataSourceObserver: DataSourceListener?
    
    private let reactionsCollection: AmityCollection<AmityReaction>
    private var reactionsToken: AmityNotificationToken?
    
    private var viewModels: [ReactionViewModel] = []
    
    init(reactionsCollection: AmityCollection<AmityReaction>) {
        self.reactionsCollection = reactionsCollection
        reactionsToken = reactionsCollection.observe { [weak self] _, _, _ in
            self?.rebuildViewModels()
            self?.dataSourceObserver?.didUpdateDataSource()
        }
    }
    
    private func rebuildViewModels() {
        viewModels = reactionsCollection.allObjects().map { reaction in
            ReactionViewModel(
                reactionName: reaction.reactionName,
                userDisplayName: reaction.reactorDisplayName,
                createdAt: reaction.createdAt
            )
        }
    }
    
    func numberOfReactions() -> Int {
        viewModels.count
    }

    func reaction(for indexPath: IndexPath) -> ReactionViewModel {
        return viewModels[indexPath.row]
    }

    func loadMore() {
        reactionsCollection.nextPage()
    }
    
}

class StaticReactionDatasource: IReactionDatasource {
    
    weak var dataSourceObserver: DataSourceListener?
    
    private let viewModels: [ReactionViewModel]

    init(viewModels: [ReactionViewModel]) {
        self.viewModels = viewModels
    }
    
    func numberOfReactions() -> Int {
        return viewModels.count
    }

    func reaction(for indexPath: IndexPath) -> ReactionViewModel {
        return viewModels[indexPath.row]
    }

    func loadMore() {
        // Intentionally left empty
    }
    
}

