//
//  ChannelRealTimeEventController.swift
//  SampleApp
//
//  Created by Amity on 11/10/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK

class ChannelRealTimeEventController: UIViewController {
    
    let tableView = UITableView()
    
    var manager: ChannelRealTimeEventManager!
    var shouldStartObserve: Bool = false
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(tableView)
        
        NSLayoutConstraint.activate([
            tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor),
            tableView.topAnchor.constraint(equalTo: view.topAnchor),
            tableView.bottomAnchor.constraint(equalTo: view.bottomAnchor)
        ])
        
        let nib = UINib(nibName: RealTimeEventHeaderCell.identifier, bundle: nil)
        tableView.register(nib, forCellReuseIdentifier: RealTimeEventHeaderCell.identifier)
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: UITableViewCell.identifier)
        
        tableView.dataSource = self
        tableView.delegate = self
        tableView.estimatedRowHeight = 100
        tableView.rowHeight = UITableView.automaticDimension
        tableView.separatorStyle = .singleLine
        tableView.separatorInset = UIEdgeInsets(top: 0, left: 24, bottom: 0, right: 24)
        tableView.reloadData()
        
        if shouldStartObserve {
            manager.observeCurrentChannel()
        }
        
        manager.observeChannelChanges = { [weak self] in
            self?.tableView.reloadSections([0], with: .automatic)
        }
    }
}

extension ChannelRealTimeEventController: UITableViewDataSource, UITableViewDelegate {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return manager.elements.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let sectionElement = manager.elements[indexPath.section]
        
        switch sectionElement {
        case .header:
            let cell = tableView.dequeueReusableCell(withIdentifier: RealTimeEventHeaderCell.identifier) as! RealTimeEventHeaderCell
            
            cell.modelDetailLabel.numberOfLines = 0
            cell.modelDetailLabel.text = manager.channel.modelDescription
            
            cell.subscribeAction = { [weak self] in
                guard let weakSelf = self else { return }
                
                self?.manager.subscribeEvent(completion: { isSuccess in
                    let message = isSuccess ? "Subscribed Successfully" : "Failed to subscribe event"
                    AppUtility.showAlert(in: weakSelf, title: "", message: message, action: nil)
                })
            }
            
            cell.unsubscribeAction = { [weak self] in
                guard let weakSelf = self else { return }
                
                self?.manager.unsubscribeEvent(completion: { isSuccess in
                    let message = isSuccess ? "Unsubscribed Successfully" : "Failed to unsubscribe event"
                    AppUtility.showAlert(in: weakSelf, title: "", message: message, action: nil)
                })
            }
            
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
    }
}
