//
//  LiveStreamCell.swift
//  SampleAppLiveVideo
//
//  Created by Nutchaphon Rewik on 24/1/2564 BE.
//  Copyright © 2564 Amity. All rights reserved.
//

import UIKit
import AmitySDK

class LiveStreamCell: UITableViewCell {
    
    @IBOutlet private weak var streamIdLabel: UILabel!
    @IBOutlet private weak var thumbnailImageView: UIImageView!
    @IBOutlet private weak var streamTitleLabel: UILabel!
    @IBOutlet private weak var streamDescriptionLabel: UILabel!
    @IBOutlet private weak var personNameLabel: UILabel!
    
    private var dequeueIdentifier = NSUUID().uuidString
    
    override func awakeFromNib() {
        super.awakeFromNib()
        thumbnailImageView.clipsToBounds = true
        thumbnailImageView.layer.cornerRadius = 8.0
        thumbnailImageView.layer.borderWidth = 1.0
        thumbnailImageView.layer.borderColor = UIColor.systemGray.cgColor
    }
    
    override func prepareForReuse() {
        super.prepareForReuse()
        dequeueIdentifier = NSUUID().uuidString
    }
    
    func configure(with streamObject: AmityStream?) {
        
        guard let streamObject = streamObject else {
            streamIdLabel.text = "-"
            streamTitleLabel.text = "Not Found"
            streamDescriptionLabel.text = "..."
            personNameLabel.text = "..."
            return
        }
        
        let description = streamObject.streamDescription ?? "No Description"
        
        let name: String
        let maxLenght = 15
        if
            let user = streamObject.user,
            let displayName = user.displayName {
            
            if displayName.count > maxLenght {
                name = displayName.prefix(maxLenght) + "..."
            } else {
                name = displayName
            }
        } else {
            name = "No user"
        }
        
        streamIdLabel.text = "id: \(streamObject.streamId)"
        streamTitleLabel.text = streamObject.title
        streamDescriptionLabel.text = description
        personNameLabel.text = " \(name) "
        
        if let thumbnail = streamObject.thumbnail {
            setThumbnailImageAsynchrnously(from: thumbnail.fileURL)
        } else {
            thumbnailImageView.image = nil
        }
        
    }
    
    // This function
    // - Asychrnously download the image and render after finish downloading.
    // - Handle cell dequeue logic.
    private func setThumbnailImageAsynchrnously(from fileUrl: String) {
        // Capture the current dequeue identifier, to check when the completion is called.
        let currentDequeueIdentifier = dequeueIdentifier
        ImageManager.shared.getThumbnailImage(fromThumbnailFileUrl: fileUrl, completion: { [weak self] image in
            guard let strongSelf = self else {
                return
            }
            if currentDequeueIdentifier != strongSelf.dequeueIdentifier {
                // This cell is dequeued to render another streamObject.
                // So it doesn't make sense to display the downloaded image now.
                return
            }
            self?.thumbnailImageView.image = image
        })
    }
    
}

extension LiveStreamCell {
    
    static func nib() -> UINib {
        return UINib(nibName: "LiveStreamCell", bundle: .main)
    }
    
    static let identifier: String = "LiveStreamCell"
    
    static let height: CGFloat = 100
    
}
